/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var BioPlates = function()
{
	var bioplates = {};
	var wellInfoCache = [];
	
	/**
		Initialize the page.
	*/
	bioplates.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			Buttons.addClickHandler('btnSave', bioplates.save);
			Buttons.addClickHandler('close', App.closeWindow);
			
			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', Annotations.autoLoadEditFrame);
			TabControl.addTabValidator('settings.info', bioplates.validateBioPlate);

			// Plate geometry
			if (Doc.element('plategeometry_id'))
			{
				Buttons.addClickHandler('plategeometry_id.select', bioplates.selectPlateGeometry);
				Events.addEventHandler('plategeometry_id', 'base-selected', bioplates.setPlateGeometryCallback);

				// Bioplate type
				Buttons.addClickHandler('bioplatetype_id.select', bioplates.selectBioPlateType);
				Events.addEventHandler('bioplatetype_id', 'base-selected', bioplates.setBioPlateTypeCallback);
				Events.addEventHandler('bioplatetype_id', 'change', bioplates.bioPlateTypeOnChange);
				bioplates.bioPlateTypeOnChange();
			}

			// Storage location
			Buttons.addClickHandler('storage_id.select', bioplates.selectStorageLocation);
			Events.addEventHandler('storage_id', 'base-selected', bioplates.setStorageLocationCallback);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'BIOPLATE', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
			Buttons.addClickHandler('btnMoveBioMaterial', bioplates.moveBioMaterial);
			Buttons.addClickHandler('btnCreateChildBioPlate', bioplates.createChildBioPlate);
			
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.overview', Overview.loadOnce);
			TabControl.addTabActivateListener('main.history', History.loadOnce);
			TabControl.addTabActivateListener('main.events', bioplates.viewEvents);
			TabControl.addTabActivateListener('main.wells', bioplates.viewWells);
			
			Events.addEventHandler('bioplate', 'mouseleave', bioplates.hideWellInfo);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'BIOPLATE'};
			var tableAttributes = {'table-id': 'bioplates'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	// Add event handlers to the 'well' icons
	bioplates.initElements = function(element, autoInit)
	{
		if (autoInit == 'well')
		{
			if (Data.int(element, 'allow-edit'))
			{
				Events.addEventHandler(element, 'click', bioplates.editWell);
			}
			if (Data.int(element, 'show-info'))
			{
				Events.addEventHandler(element, 'mouseover', bioplates.showWellInfo);
			}
			else
			{
				Events.addEventHandler(element, 'mouseover', bioplates.hideWellInfo);
			}
		}
	}

	
	bioplates.viewWells = function()
	{
		var bioplateId = Data.get('page-data', 'item-id');
		var url = 'wells/index.jsp?ID='+App.getSessionId();
		url += '&bioplate_id='+bioplateId;
		location.replace(url);
	}
	
	bioplates.viewEvents = function()
	{
		var bioplateId = Data.get('page-data', 'item-id');
		var url = 'events/index.jsp?ID='+App.getSessionId();
		url += '&bioplate_id='+bioplateId;
		location.replace(url);
	}
	
	bioplates.moveBioMaterial = function()
	{
		var bioplateId = Data.get('page-data', 'item-id');
		var url = 'index.jsp?ID='+App.getSessionId();
		url += '&cmd=MoveBioMaterial';
		url += '&item_id='+bioplateId;
		Dialogs.openPopup(url, 'MoveBioMaterial', 900, 600);
	}
	
	bioplates.createChildBioPlate = function()
	{
		var bioplateId = Data.get('page-data', 'item-id');
		var url = 'index.jsp?ID='+App.getSessionId();
		url += '&cmd=CreateChildBioPlate';
		url += '&item_id='+bioplateId;
		Dialogs.openPopup(url, 'CreateChildBioPlate', 900, 600);
	}
	
	bioplates.editWell = function(event)
	{
		var well = event.currentTarget;
		var wellId = Data.get(well, 'item-id');
		Items.editItem('BIOWELL', wellId);
	}
	
	bioplates.showWellInfo = function(event)
	{
		var well = event.currentTarget;
		var wellId = Data.get(well, 'item-id');
		var coordinate = Data.get(well, 'coordinate');
		
		// Display the 'wellInfo' postit note
		var wellInfo = Doc.element('wellInfo');
		var wellTd = Doc.element('well.' + wellId);
		var contentPos = Doc.getElementPosition('main.content');
		var pos = Doc.getElementPosition(wellTd);
		wellInfo.style.left = (pos.left + pos.width - 5 - contentPos.left) + 'px';
		wellInfo.style.top = (pos.top + 10-contentPos.top) + 'px';
		Doc.show('wellInfo');

		// Get the well information
		var info = wellInfoCache['well.'+wellId];
		if (!info)
		{
			// Use Ajax to load the information
			wellInfo.innerHTML = '[' + coordinate + '] loading, please wait...';
			info = bioplates.loadWellInfo(wellId);
		}
		var html;
		var bioMaterial = info.bioMaterial;
		if (bioMaterial)
		{
			if (bioMaterial.denied)
			{
				html = 'Denied';
			}
			else
			{
				var id = bioMaterial.id;
				var type = bioMaterial.type;
				var subtype = bioMaterial.subtype;
				html = '<span class="link"';
				html += ' data-item-type="'+type+'" data-item-id="'+id+'"';
				html += ' data-no-edit="'+bioMaterial.editable ? 0 : 1+'"';
				html += ' title="View this item';
				if (bioMaterial.editable) html += ' (use CTRL, ALT or SHIFT to edit)';				
				html += '">'+bioMaterial.name+'</span>';
				html += ': <b>' + type + '</b>';
				if (subtype) html += ' <span class="itemsubtype">[' + subtype.name + ']</span>';
				html += '<br>'+bioMaterial.description;
			}
		}
		else
		{
			html = 'Empty';
		}
		wellInfo.innerHTML = html;
		Events.addEventHandler(wellInfo.getElementsByClassName('link')[0], 'click', Items.itemOnClick);
	}

	bioplates.hideWellInfo = function()
	{
		Doc.hide('wellInfo');
	}

	bioplates.loadWellInfo = function(wellId)
	{
		var request = Ajax.getXmlHttpRequest();
		var url = 'wells/ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=WellInfo&encodeStrings=1&item_id=' + wellId;
		request.open("GET", url, false); 
		
		// NOTE! 'false' causes code to wait for the response. aka. 'Synchronous AJAX' or SJAX.
		request.send(null);
		var response = JSON.parse(request.responseText);
		if (response.status != 'ok')
		{
			alert(response.message);
			return false;
		}

		wellInfoCache['well.'+wellId] = response;
		return response;
	}

	
	bioplates.validateBioPlate = function()
	{
		var frm = document.forms['bioplate'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (frm.bioplatetype_id && frm.bioplatetype_id.length == 0)
		{
			Forms.showNotification('bioplatetype_id.select', 'You must select a bioplate type');
			return false;
		}
		if (frm.plategeometry_id && frm.plategeometry_id.length == 0)
		{
			Forms.showNotification('plategeometry_id.select', 'You must select a plate geometry');
			return false;
		}
		return true;
	}

	// Submit the form
	bioplates.save = function()
	{
		var frm = document.forms['bioplate'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			frm.submit();
		}
	}

	bioplates.selectPlateGeometry = function()
	{
		var frm = document.forms['bioplate'];
		var url = '';
		if (frm.plategeometry_id.length > 0) 
		{
			url += '&item_id='+frm.plategeometry_id[0].value;
		}
		Dialogs.selectItem('PLATEGEOMETRY', 'plategeometry_id', 0, url);
	}
	
	bioplates.setPlateGeometryCallback = function(event)
	{
		var frm = document.forms['bioplate'];
		var list = frm.plategeometry_id;
		if (list.length < 1 || list[0].disabled)
		{
			Forms.addListOption(list, 0, new Option());
		}
		list[0].value = event.detail.id;
		list[0].text = event.detail.name;
		list.selectedIndex = 0;
	}

	bioplates.selectBioPlateType = function()
	{
		var frm = document.forms['bioplate'];
		var url = '';
		if (frm.bioplatetype_id.length > 0) 
		{
			url += '&item_id='+frm.bioplatetype_id[0].value;
		}
		Dialogs.selectItem('BIOPLATETYPE', 'bioplatetype_id', 0, url);		
	}
	
	bioplates.setBioPlateTypeCallback = function(event)
	{
		
		var frm = document.forms['bioplate'];
		var list = frm.bioplatetype_id;
		if (list.length < 1 || list[0].disabled)
		{
			Forms.addListOption(list, 0, new Option());
		}
		list[0].value = event.detail.id;
		list[0].text = event.detail.name;
		list.selectedIndex = 0;
		bioplates.bioPlateTypeOnChange();
	}

	bioplates.bioPlateTypeOnChange = function()
	{
		var frm = document.forms['bioplate'];
		if (frm.bioplatetype_id.selectedIndex < 0) return;
		
		var bioPlateTypeInfo = bioplates.getBioPlateTypeInfo(frm.bioplatetype_id.value);
		var freezerTypeId = Data.get('storage_id', 'freezer-id');
		var storageTypeId = bioPlateTypeInfo.storageType ?  bioPlateTypeInfo.storageType.id : freezerTypeId;
		Data.set('storage_id', 'storage-type-id', storageTypeId);
		var recentInfo = ItemSubtype.getProjectDefaultAndRecentItems('BIOPLATE', storageTypeId);
		ItemSubtype.updateSelectionList(frm.storage_id, recentInfo['recent'], recentInfo['default']);
	}
	
	
	bioplates.getBioPlateTypeInfo = function(bioPlateTypeId)
	{
		var request = Ajax.getXmlHttpRequest();
		var url = '../bioplatetypes/ajax.jsp?ID=' + getSessionId();
		url += '&cmd=GetPlateTypeInfo';
		url += '&item_id='+bioPlateTypeId;
		request.open("GET", url, false); 
		request.send(null);
		return JSON.parse(request.responseText);
	}
	
	bioplates.selectStorageLocation = function()
	{
		var frm = document.forms['bioplate'];
		var url = '';
		if (frm.storage_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.storage_id[1].value));				
			url += '&item_id='+id;
		}
		var storageTypeId = Data.get('storage_id', 'storage-type-id', Data.get('storage_id', 'freezer-id'));
		url += '&resetTemporary=1&tmpfilter:INT:itemSubtype='+storageTypeId;
		Dialogs.selectItem('HARDWARE', 'storage_id', 0, url);
	}
	
	bioplates.setStorageLocationCallback = function(event)
	{
		var frm = document.forms['bioplate'];
		if (frm.storage_id.length < 1)
		{
			frm.storage_id[frm.storage_id.length] = new Option();
		}
		frm.storage_id[0].value = event.detail.id;
		frm.storage_id[0].text = event.detail.name;
		frm.storage_id.selectedIndex = 0;
	}

	return bioplates;
}();

Doc.addElementInitializer(BioPlates.initElements);
Doc.onLoad(BioPlates.initPage);
