/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var BioPlateTypes = function()
{
	var bioplatetypes = {};
	var subtypes;
	
	/**
		Initialize the page.
	*/
	bioplatetypes.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', bioplatetypes.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabValidator('settings.info', bioplatetypes.validateBioPlateType);
			
			// Biomaterial type
			subtypes = Data.json('page-data', 'subtypes');
			Events.addEventHandler('bioMaterialType', 'change', bioplatetypes.bioMaterialTypeOnChange);
			bioplatetypes.bioMaterialTypeOnChange();
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'BIOPLATETYPE', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'BIOPLATETYPE'};
			var tableAttributes = {'table-id': 'bioplatetypes'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	bioplatetypes.validateBioPlateType = function()
	{
		var frm = document.forms['bioPlateType'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}

	// Submit the form
	bioplatetypes.save = function()
	{
		var frm = document.forms['bioPlateType'];
		if (TabControl.validateActiveTab('settings'))
		{
			frm.submit();
		}
	}

	bioplatetypes.bioMaterialTypeOnChange = function()
	{
		var frm = document.forms['bioPlateType'];
		var bmType = frm.bioMaterialType ? frm.bioMaterialType.value : Data.get('page-data', 'current-bm-type');
		
		var currentSubtypeId = Data.int('page-data', 'current-subtype');
		frm.subtype_id.length = 0;
		frm.subtype_id[0] = new Option('-any-', 0);
		
		for (var i = 0; i < subtypes.length; i++)
		{
			var subtype = subtypes[i];
			if (subtype.bmType == bmType)
			{
				var opt = new Option(subtype.name, subtype.id, false, currentSubtypeId == subtype.id);
				if (subtype.description) opt.title = subtype.description;
				frm.subtype_id[frm.subtype_id.length] = opt;
			}
		}
	}
	

	return bioplatetypes;
}();

Doc.onLoad(BioPlateTypes.initPage);
