/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var DataFiles = function()
{
	var datafiles = {};
	var internal = {};
	
	var fileTypes;
	var fileActions = [];
	var visibleRecentFilesDiv;
	
	var editFrameLoaded = false;
	var lastPlatformId = null;
	var lastVariantId = null;
	var lastSubtypeId = null;
	
	datafiles.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'select-datafiles')
		{
			// Parse the current filetype information
			fileTypes = Data.json('page-data', 'filetypes');
			for (var i = 0; i < fileTypes.length; i++)
			{
				var fileType = fileTypes[i];
				fileTypes['ID'+fileType.id] = fileType;
				
				var attributes = {'filetype-id': fileType.id};
				// 'Browse' button, click to open file dialog, 'base-selected' to receive selected files
				Events.addEventHandler('browsefile.'+fileType.id, 'click', datafiles.browseOnClick, attributes);
				Events.addEventHandler('browsefile.'+fileType.id, 'base-selected', datafiles.onFileSelected);

				// 'Add' button, click to open file dialog, 'base-selected' to receive selected files
				Events.addEventHandler('addfile.'+fileType.id, 'click', datafiles.addOnClick, attributes);
				Events.addEventHandler('addfile.'+fileType.id, 'base-selected', datafiles.onFileSelected);

				// Remove file handler is attached to file list, but need target file
				Events.addEventHandler('filelist.'+fileType.id, 'click', datafiles.removeOnClick, attributes);
				
				// Show recent files list, and handle recent file selection
				Events.addEventHandler('showrecent.'+fileType.id, 'click', datafiles.showRecentFilesOnClick, attributes);
				Events.addEventHandler('recentfiles.'+fileType.id, 'click', datafiles.recentFileOnClick, attributes);
				
				//Events.addEventHandler('addfile.'+fileType.id, 'click', datafiles.addOnClick, attributes);
			}

			// Hide 'recently used files' list if visible
			Events.addEventHandler(document.body, 'click', datafiles.hideVisibleRecentFilesDiv);
		}
	}
	
	/*
		Event handler for showing the list of recently used files for
		a given data file type.
	*/
	datafiles.showRecentFilesOnClick = function(event)
	{
		var target = event.currentTarget;
		var fileTypeId = Data.get(target, 'filetype-id');
		
		var recentDiv = Doc.element('recentfiles.'+fileTypeId);
		var currentDiv = visibleRecentFilesDiv;
		datafiles.hideVisibleRecentFilesDiv();
		if (currentDiv == recentDiv) return;
		
		var fileContainer = Doc.element('container.'+fileTypeId);
		var pos = Doc.getElementPosition(fileContainer, true);
		Doc.show('recentfiles.'+fileTypeId);
			
		recentDiv.style.top = (pos.bottom-2) + 'px';
		recentDiv.style.left = (pos.left) + 'px';
		recentDiv.style.width = (pos.width-2) + 'px';

		visibleRecentFilesDiv = recentDiv;
		event.stopPropagation();
	}
	
	/*
		Hide the currently visible 'recently used files' list.
	 */
	datafiles.hideVisibleRecentFilesDiv = function()
	{
		if (visibleRecentFilesDiv)
		{
			visibleRecentFilesDiv.style.display = 'none';
			visibleRecentFilesDiv = null;
		}
	}
	
	/*
		Event handler when a user selects a recently used file. The
		event handler is attached to the list container, but the target is
		an individual file entry.
	 */
	datafiles.recentFileOnClick = function(event)
	{
		var fileTarget = event.target;
		var fileTypeTarget = event.currentTarget;
		
		var fileTypeId = Data.get(fileTypeTarget, 'filetype-id');
		var fileType = fileTypes['ID'+fileTypeId];

		var file = {};
		file.id = Data.int(fileTarget, 'file-id');
		file.name = file.id ? Doc.element('recentfile.'+fileTypeId+'.'+file.id).innerHTML : '';
		
		if (fileType.allowMultiple && file.id)
		{
			internal.addFile(fileType, file);
		}
		else
		{
			internal.setFile(fileType, file);
		}
	}
	
	datafiles.browseOnClick = function(event)
	{
		var target = event.currentTarget;
		var fileTypeId = Data.get(target, 'filetype-id');
		var fileType = fileTypes['ID'+fileTypeId];
		
		var frm = document.forms['datafiles'];
		var url = '../../filemanager/index.jsp?ID=' + App.getSessionId();
		url += '&cmd=SelectOne&callback='+target.id;
		if (fileType.extension)
		{
			url += '&resetTemporary=1&tmpfilter:STRING:name='+escape('%.' + fileType.extension);
		}
		else
		{
			url += '&resetTemporary=1&filter:STRING:name=';
		}
		
		Dialogs.openPopup(url, 'SelectFile', 1050, 700);
		event.stopPropagation();
	}
	
	datafiles.onFileSelected = function(event)
	{
		var target = event.currentTarget;
		var fileTypeId = Data.get(target, 'filetype-id');
		var fileType = fileTypes['ID'+fileTypeId];
		var file = event.detail;
		if (fileType.allowMultiple)
		{
			internal.addFile(fileType, file);
		}
		else
		{
			internal.setFile(fileType, file);
		}
	}
	
	internal.setFile = function(fileType, file)
	{
		var frm = document.forms['datafiles'];
		var fileDiv = Doc.element('filelist.'+fileType.id);
		if (file.id)
		{
			fileDiv.innerHTML = '<div id="filelist.'+fileType.id+'.'+file.id+'" class="file">'+file.name+'</div>';
		}
		else
		{
			fileDiv.innerHTML = '';
		}
		if (fileType.hasValidator) frm['datafiles.validate'].checked = true;
		internal.registerFileAction('setfile', fileType.id, file.id);
	}
	
	datafiles.addOnClick = function(event)
	{
		var target = event.currentTarget;
		var fileTypeId = Data.get(target, 'filetype-id');
		var fileType = fileTypes['ID'+fileTypeId];

		var frm = document.forms['datafiles'];
		var url = '../../filemanager/index.jsp?ID=' + App.getSessionId();
		url += '&cmd=SelectMultiple&callback='+target.id;
		if (fileType.extension)
		{
			url += '&resetTemporary=1&tmpfilter:STRING:name='+escape('%.' + fileType.extension);
		}
		else
		{
			url += '&resetTemporary=1&filter:STRING:name=';
		}
		Dialogs.openPopup(url, 'SelectFiles', 1050, 700);
		event.stopPropagation();
	}
	
	internal.addFile = function(fileType, file)
	{
		var frm = document.forms['datafiles'];
		var fileDiv = Doc.element('filelist.'+fileType.id + '.' + file.id);
		if (fileDiv) return; // This file has already been selected
		
		fileDiv = document.createElement('div');
		fileDiv.id = 'filelist.'+fileType.id+'.'+file.id;
		
		var html = file.name;
		html += ' <span id="remove.'+fileType.id+'.'+file.id +'" class="icon link" title="Remove this file"';
		html += ' data-file-id="'+file.id+'">';
		html += '<img src="' + App.getRoot() + 'images/remove.png"></span>';
		
		fileDiv.innerHTML = html;
		Doc.element('filelist.'+fileType.id).appendChild(fileDiv);
		if (fileType.hasValidator) frm['datafiles.validate'].checked = true;
		internal.registerFileAction('addfile', fileType.id, file.id);
	}

	datafiles.removeOnClick = function(event)
	{
		var fileTarget = event.target.parentNode;
		var fileId = Data.get(fileTarget, 'file-id');
		if (!fileId) return;
		
		var fileTypeTarget = event.currentTarget;
		var fileTypeId = Data.get(fileTypeTarget, 'filetype-id');
		var fileType = fileTypes['ID'+fileTypeId];
		
		var fileDiv = Doc.element('filelist.'+fileTypeId+'.'+fileId);
		Doc.element('filelist.'+fileTypeId).removeChild(fileDiv);
		if (fileType.hasValidator) frm['datafiles.validate'].checked = true;
		internal.registerFileAction('removefile', fileTypeId, fileId);
		event.stopPropagation();
	}
	
	datafiles.autoLoadEditFrame = function(platformId, variantId, subtypeId)
	{
		if (editFrameLoaded && lastPlatformId == platformId && lastVariantId == variantId && lastSubtypeId == subtypeId) 
		{
			return;
		}
		lastPlatformId = platformId;
		lastVariantId = variantId;
		lastSubtypeId = subtypeId;
		editFrameLoaded = true;

		var url = App.getRoot()+'common/datafiles/select_files.jsp?ID='+App.getSessionId();
		url += '&item_type='+Data.get('datafiles-data', 'item-type');
		url += '&item_id='+Data.get('datafiles-data', 'item-id');
		if (platformId) url += '&platform_id='+platformId;
		if (variantId) url += '&variant_id='+variantId;
		if (subtypeId) url += '&itemsubtype_id='+subtypeId;
		frames['datafiles'].location.replace(url);
	}
	
	/*
		Write current actions to form. If called from outside, 
		re-direct call to 'datafiles' iframe.
	 */
	datafiles.writeFileActionsToForm = function(toForm)
	{
		var pageId = Doc.getPageId();
		if (pageId != 'select-datafiles')
		{
			if (!editFrameLoaded) return;
			frames['datafiles'].DataFiles.writeFileActionsToForm(toForm);
			return;
		}
		
		var frm = document.forms['datafiles'];
		if (frm['datafiles.validate'])
		{
			Forms.addHidden(toForm, 'datafiles.validate', frm['datafiles.validate'].checked ? 1 : 0);
		}
		for (var i = 0; i < fileActions.length; i++)
		{
			var action = fileActions[i];
			Forms.addHidden(toForm, action.cmd + '.' + action.fileTypeId, action.fileId);
		}
	}
	
	/*
		Register a new 'file action' for a given filetype and file.
		The action cmd can be one of: 
	 */
	internal.registerFileAction = function(cmd, fileTypeId, fileId)
	{
		// Check if other actions should be removed as a result of this one
		for (var i = fileActions.length-1; i >= 0; i--)
		{
			var other = fileActions[i];
			var remove = false;
			if (cmd == 'setfile')
			{
				// Remove all other actions for the given file type
				remove = other.fileTypeId == fileTypeId;
			}
			else
			{
				// Remove all other actions for the given file type+file
				remove = other.fileTypeId == fileTypeId && other.fileId == fileId;
			}
			if (remove)	fileActions.splice(i, 1);
		}
		
		// Create and store the action
		var action = {};
		action.cmd = cmd;
		action.fileTypeId = fileTypeId,
		action.fileId = fileId;
		fileActions[fileActions.length] = action;
	}
	
	return datafiles;
}();

Doc.onLoad(DataFiles.initPage);
