/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Share = function()
{
	var EVERYONE_ID;
	var SHARE_TO_EVERYONE;
	var Permission = {};
	
	var share = {};
	
	share.initPage = function()
	{
		Permission.READ = Data.int('page-data', 'read');
		Permission.USE = Data.int('page-data', 'use');
		Permission.WRITE = Data.int('page-data', 'write');
		Permission.DELETE = Data.int('page-data', 'delete');
		Permission.SET_OWNER = Data.int('page-data', 'set-owner');
		Permission.SET_PERMISSION = Data.int('page-data', 'set-permission');

		// Dialog buttons
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnSave', share.save);
		
		Buttons.addClickHandler('btnAddUsers', share.addUsers);
		Events.addEventHandler('btnAddUsers', 'base-selected', share.addUserCallback);
		Buttons.addClickHandler('btnAddGroups', share.addGroups);
		Events.addEventHandler('btnAddGroups', 'base-selected', share.addGroupCallback);
		Buttons.addClickHandler('btnAddProjects', share.addProjects);
		Events.addEventHandler('btnAddProjects', 'base-selected', share.addProjectCallback);
		Buttons.addClickHandler('btnAddPermissionTemplates', share.addPermissionTemplates);
		Events.addEventHandler('btnAddPermissionTemplates', 'base-selected', share.addPermissionTemplateCallback);

		// Permission checkboxes
		Events.addEventHandler('read', 'click', share.permissionsOnClick);
		Events.addEventHandler('use', 'click', share.permissionsOnClick);
		Events.addEventHandler('write', 'click', share.permissionsOnClick);
		Events.addEventHandler('delete', 'click', share.permissionsOnClick);
		Events.addEventHandler('setOwner', 'click', share.permissionsOnClick);
		Events.addEventHandler('setPermission', 'click', share.permissionsOnClick);
		Events.addEventHandler('members', 'change', share.membersOnChange);

		EVERYONE_ID = Data.int('page-data', 'everyone-id');
		SHARE_TO_EVERYONE = Data.int('page-data', 'share-to-everyone');
	}

	/**
		Event handler that update the permission checkboxes depending
		on the currently selected item in the list.
	*/
	share.membersOnChange = function()
	{
		var frm = document.forms['share'];
		var item = frm.members[frm.members.selectedIndex].item;
		if (item && item.id)
		{
			var isTemplate = item.type == 'PERMISSIONTEMPLATE';
			var isEveryone = item.id == EVERYONE_ID && item.type == 'GROUP';
			share.checkPermissions(item.value, isEveryone, isTemplate);
		}
	}

	/**
		Event handler for the permission checkboxes that modify the permissions
		on all selected users/groups/projects in the list
	*/
	share.permissionsOnClick = function(event)
	{
		var target = event.currentTarget;
		var frm = document.forms['share'];
		
		share.correctPermissions(target.checked);
		var permissionCode = share.getPermissionCode();
		var permissionText = share.getPermissionText(permissionCode)
		for (var i = 0; i < frm.members.length; i++)
		{
			var option = frm.members[i];
			if (option.selected && option.item && option.item.id)
			{
				var item = option.item;
				var disabled = item.type == 'PERMISSIONTEMPLATE' || (item.id == EVERYONE_ID && item.type == 'GROUP' && !SHARE_TO_EVERYONE);
				if (!disabled)
				{
					option.item.value = permissionCode;
					var text = option.text.replace(/\[.*\]/, '['+permissionText+']');
					option.text = text;
				}
			}
		}
	}

	/**
		Collected the modified permissions and submit the form.
	*/
	share.save = function()
	{
		var frm = document.forms['share'];
		Link.exportActions('members', '!');
		frm.submit();
	}

	/**
		Open a popup dialog for selecting users.
	*/
	share.addUsers = function(event)
	{
		var users = Link.getIdsInList('members', 'USER');
		var url = '&exclude='+users.join(',');
		url += '&permission=READ';
		Dialogs.selectItem('USER', event.currentTarget.id, 1, url);
	}
	
	/**
		Callback that add more users to the list.
	*/
	share.addUserCallback = function(event)
	{
		var permissionCode = share.getPermissionCode();
		var permissionString = share.getPermissionText(permissionCode);
		
		event.detail.name += ' ['+permissionString+']';
		event.detail.value = permissionCode;
		
		Link.addItem('members', 'USER', event.detail);
		share.membersOnChange();
	}

	/**
		Open a popup dialog for selecting groups.
	*/
	share.addGroups = function(event)
	{
		var groups = Link.getIdsInList('members', 'GROUP');
		if (!SHARE_TO_EVERYONE)
		{
			groups[groups.length] = EVERYONE_ID;
		}
		var url = '&exclude='+groups.join(',');
		url += '&permission=READ';
		Dialogs.selectItem('GROUP', event.currentTarget.id, 1, url);
	}
	
	/**
		Callback that add more groups to the list.
	*/
	share.addGroupCallback = function(event)
	{
		var permissionCode = share.getPermissionCode();
		var permissionString = share.getPermissionText(permissionCode);
		
		event.detail.name += ' ['+permissionString+']';
		event.detail.value = permissionCode;
		
		Link.addItem('members', 'GROUP', event.detail);
		share.membersOnChange();
	}

	/**
		Open a popup dialog for selecting projects.
	*/
	share.addProjects = function(event)
	{
		var projects = Link.getIdsInList('members', 'PROJECT');
		var url = '&exclude='+projects.join(',');
		Dialogs.selectItem('PROJECT', event.currentTarget.id, 1, url);
	}
	
	/**
		Callback that add more projects to the list.
	*/
	share.addProjectCallback = function(event)
	{
		var permissionCode = share.getPermissionCode();
		var permissionString = share.getPermissionText(permissionCode);
		
		event.detail.name += ' ['+permissionString+']';
		event.detail.value = permissionCode;
		
		Link.addItem('members', 'PROJECT', event.detail);
		share.membersOnChange();
	}
	
	/**
		Open a popup dialog for selecting permission templates.
	*/
	share.addPermissionTemplates = function(event)
	{
		var templates = Link.getIdsInList('members', 'PERMISSIONTEMPLATE');
		var url = '&exclude='+templates.join(',');
		Dialogs.selectItem('PERMISSIONTEMPLATE', event.currentTarget.id, 1, url);
	}
	
	/**
		Callback that add more permission templates to the list.
	*/
	share.addPermissionTemplateCallback = function(event)
	{
		Link.addItem('members', 'PERMISSIONTEMPLATE', event.detail);
		share.membersOnChange();
	}

	/**
		Get a textual description of the permissions that are included in
		the given permission code.
	*/
	share.getPermissionText = function(permissionCode)
	{
		var s = '';
		if (permissionCode > 0)
		{
			s += permissionCode & Permission.READ ? "R" : "-";
			s += permissionCode & Permission.USE ? "U" : "-";
			s += permissionCode & Permission.WRITE ? "W" : "-";
			s += permissionCode & Permission.DELETE ? "D" : "-";
			s += permissionCode & Permission.SET_OWNER ? "O" : "-";
			s += permissionCode & Permission.SET_PERMISSION ? "P" : "-";
		}
		else
		{
			s = 'varying';
		}
		return s;
	}
	
	/**
		Get the permission code from the checked checkboxes.
	*/
	share.getPermissionCode = function()
	{
		var c = 0;
		var frm = document.forms['share'];
		if (frm['read'].checked) c += Permission.READ;
		if (frm['use'].checked) c += Permission.USE;
		if (frm['write'].checked) c += Permission.WRITE;
		if (frm['delete'].checked) c += Permission.DELETE;
		if (frm['set_owner'].checked) c += Permission.SET_OWNER;
		if (frm['set_permission'].checked) c += Permission.SET_PERMISSION;
		return c;
	}
	
	/**
		The permissions are related and we need to check or uncheck permissions
		depending on what was currently changed.
	*/
	share.correctPermissions = function(checked)
	{
		var frm = document.forms['share'];
		if (checked)
		{
			frm['write'].checked = frm['write'].checked || frm['delete'].checked || frm['set_owner'].checked || frm['set_permission'].checked;
			frm['use'].checked = frm['use'].checked || frm['write'].checked;
			frm['read'].checked = frm['read'].checked || frm['use'].checked;
		}
		else
		{
			frm['use'].checked = frm['use'].checked && frm['read'].checked;
			frm['write'].checked = frm['write'].checked && frm['use'].checked;
			frm['delete'].checked = frm['delete'].checked && frm['write'].checked;
			frm['set_owner'].checked = frm['set_owner'].checked && frm['write'].checked;
			frm['set_permission'].checked = frm['set_permission'].checked && frm['write'].checked;
		}
	}

	/**
		Update the permission checkboxes so that they reflect the given
		permission code. Permissions from teplates or the EVERYONE group
		may disable the permissions.
	*/
	share.checkPermissions = function(permissionCode, isEveryone, isTemplate)
	{
		var frm = document.forms['share'];
		frm['read'].checked = permissionCode & Permission.READ;
		frm['use'].checked = permissionCode & Permission.USE;
		frm['write'].checked = permissionCode & Permission.WRITE;
		frm['delete'].checked = permissionCode & Permission.DELETE;
		frm['set_owner'].checked = permissionCode & Permission.SET_OWNER;
		frm['set_permission'].checked = permissionCode & Permission.SET_PERMISSION;

		var disabled = isTemplate || (isEveryone && !SHARE_TO_EVERYONE);
		frm['read'].disabled = disabled;
		frm['use'].disabled = disabled;
		frm['write'].disabled = disabled;
		frm['delete'].disabled = disabled;
		frm['set_owner'].disabled = disabled;
		frm['set_permission'].disabled = disabled;
		if (isEveryone && !SHARE_TO_EVERYONE)
		{
			Doc.show('share_disabled');
		}
		else
		{
			Doc.hide('share_disabled');
		}
	}
	
	return share;
}();

Doc.onLoad(Share.initPage);


