/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var ArrayBatches = function()
{
	var arrayBatches = {};
	
	/**
		Initialize the page.
	*/
	arrayBatches.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', arrayBatches.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', arrayBatches.loadAnnotationsFrame);
			TabControl.addTabActivateListener('settings.inheritedAnnotations', arrayBatches.loadInheritedAnnotationsFrame);
			TabControl.addTabValidator('settings.info', arrayBatches.validateArrayBatch);

			// Array design
			Buttons.addClickHandler('arraydesign_id.select', arrayBatches.selectArrayDesign);
			Events.addEventHandler('arraydesign_id', 'base-selected', arrayBatches.setArrayDesignCallback);

			// Protocol
			Buttons.addClickHandler('protocol_id.select', arrayBatches.selectProtocol);
			Events.addEventHandler('protocol_id', 'base-selected', arrayBatches.setProtocolCallback);

			// Print robot
			Buttons.addClickHandler('printrobot_id.select', arrayBatches.selectPrintRobot);
			Events.addEventHandler('printrobot_id', 'base-selected', arrayBatches.setPrintRobotCallback);

		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'ARRAYBATCH', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnNewSlide', arrayBatches.newSlide, attributes);
			Buttons.addClickHandler('btnNewSlideWizard', arrayBatches.newSlideWizard, attributes);
			
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.overview', Overview.loadOnce);
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'ARRAYBATCH'};
			var tableAttributes = {'table-id': 'batches'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	arrayBatches.initElements = function(element, autoInit)
	{
		if (autoInit == 'new-slide')
		{
			Buttons.addClickHandler(element, arrayBatches.newSlide);
		}
		else if (autoInit == 'new-slide-wizard')
		{
			Buttons.addClickHandler(element, arrayBatches.newSlideWizard);
		}
		
	}
	
	arrayBatches.newSlide = function(event)
	{
		var batchId = Data.get(event.currentTarget, 'item-id');
		Items.newItem('ARRAYSLIDE', '&arraybatch_id='+batchId);
	}
	
	arrayBatches.newSlideWizard = function(event)
	{			
		var batchId = Data.get(event.currentTarget, 'item-id');
		var url = '../arrayslides/index.jsp?ID='+App.getSessionId();
		url += '&cmd=NewItems&arraybatch_id='+batchId;
		Dialogs.openPopup(url, 'NewArraySlides', 750, 500);
	}
	
	arrayBatches.validateArrayBatch = function()
	{
		var frm = document.forms['batch'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (frm.arraydesign_id && !frm.arraydesign_id.value)
		{
			Forms.showNotification('arraydesign_id.select', 'You must select an array design');
			return false;
		}
		return true;
	}

	// Submit the form
	arrayBatches.save = function()
	{
		var frm = document.forms['batch'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			Annotations.saveInheritedAnnotationsToForm(frm);
			frm.submit();
		}
	}
	
	arrayBatches.loadAnnotationsFrame = function()
	{
		Annotations.autoLoadEditFrame(arrayBatches.getProtocolId());
	}
	
	arrayBatches.loadInheritedAnnotationsFrame = function()
	{
		Annotations.autoLoadInheritFrame(arrayBatches.getParents());
	}

	arrayBatches.getProtocolId = function()
	{
		var frm = document.forms['batch'];
		var protocolId = 0;
		if (frm.protocol_id.length > 0 && !frm.protocol_id.disabled) 
		{
			protocolId = Math.abs(parseInt(frm.protocol_id.value));				
		}
		return protocolId;
	}

	arrayBatches.getParents = function()
	{
		var frm = document.forms['batch'];
		var parents = new Array();
		if (frm.arraydesign_id)
		{
			var arrayDesignId = Math.abs(parseInt(frm.arraydesign_id.value));
			if (arrayDesignId > 0) parents[parents.length] = 'ARRAYDESIGN:'+arrayDesignId;
		}
		return parents;
	}

	arrayBatches.selectArrayDesign = function()
	{
		var frm = document.forms['batch'];
		var url = '';
		if (frm.arraydesign_id.length > 0) 
		{
			var id = Math.abs(parseInt(frm.arraydesign_id[0].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('ARRAYDESIGN', 'arraydesign_id', 0, url);
	}
	
	arrayBatches.setArrayDesignCallback = function(event)
	{
		var frm = document.forms['batch'];
		var list = frm.arraydesign_id;
		if (list.length < 1 || list[0].value == '0')
		{
			Forms.addListOption(list, 0, new Option());
		}
		list[0].value = event.detail.id;
		list[0].text = event.detail.name;
		list.selectedIndex = 0;
	}

	arrayBatches.selectProtocol = function()
	{
		var frm = document.forms['batch'];
		var url = '';
		if (frm.protocol_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.protocol_id[1].value));				
			url += '&item_id='+id;
		}
		url += '&resetTemporary=1&tmpfilter:INT:itemSubtype='+Data.get('page-data', 'printing-protocol');
		Dialogs.selectItem('PROTOCOL', 'protocol_id', 0, url);
	}
	
	arrayBatches.setProtocolCallback = function(event)
	{
		var frm = document.forms['batch'];
		var list = frm.protocol_id;
		if (list.length < 2 || list[1].value == '0') 
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}
	
	arrayBatches.selectPrintRobot = function()
	{
		var frm = document.forms['batch'];
		var url = '';
		if (frm.printrobot_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.printrobot_id[1].value));				
			url += '&item_id='+id;
		}
		url += '&resetTemporary=1&tmpfilter:INT:itemSubtype='+Data.get('page-data', 'print-robot');
		Dialogs.selectItem('HARDWARE', 'printrobot_id', 0, url);
	}
	
	arrayBatches.setPrintRobotCallback = function(event)
	{
		var frm = document.forms['batch'];
		var list = frm.printrobot_id;
		if (list.length < 2 || list[1].value == '0')
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}
	
	return arrayBatches;
}();

Doc.onLoad(ArrayBatches.initPage);
Doc.addElementInitializer(ArrayBatches.initElements);
