/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var BioAssays = function()
{
	var bioassays = {};
	
	/**
		Initialize the page.
	*/
	bioassays.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', bioassays.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', bioassays.loadAnnotationsFrame);
			TabControl.addTabActivateListener('settings.inheritedAnnotations', bioassays.loadInheritedAnnotationsFrame);
			TabControl.addTabValidator('settings.info', bioassays.validatePhysicalBioAssay);
			
			// Size
			Events.addEventHandler('size', 'keypress', Events.integerOnly);
			
			// Protocol
			Buttons.addClickHandler('protocol_id.select', bioassays.selectProtocol);
			Events.addEventHandler('protocol_id', 'base-selected', bioassays.setProtocolCallback);

			// Subtype
			Events.addEventHandler('subtype_id', 'change', bioassays.subtypeOnChange);

			// Hardware
			Buttons.addClickHandler('hardware_id.select', bioassays.selectHardware);
			Events.addEventHandler('hardware_id', 'base-selected', bioassays.setHardwareCallback);
			
			// Array slide
			Buttons.addClickHandler('arrayslide_id.select', bioassays.selectArraySlide);
			Events.addEventHandler('arrayslide_id', 'base-selected', bioassays.setArraySlideCallback);
			Events.addEventHandler('arrayslide_id', 'change', bioassays.arraySlideOnChange);
			
			// Parent extracts
			Events.addEventHandler('extracts', 'change', bioassays.extractsOnChange);
			Buttons.addClickHandler('btnAddExtracts', bioassays.addExtracts);
			Events.addEventHandler('btnAddExtracts', 'base-selected', bioassays.addExtractCallback);
			
			// Used quantity and position
			Events.addEventHandler('used_quantity', 'keypress', Events.numberOnly);
			Events.addEventHandler('used_quantity', 'keyup', bioassays.usedQuantityOnBlur);
			Events.addEventHandler('position', 'keypress', Events.integerOnly);
			Events.addEventHandler('position', 'keyup', bioassays.usedQuantityOnBlur);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'PHYSICALBIOASSAY', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnNewDerivedBioAssay', bioassays.newDerivedBioAssay, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.overview', Overview.loadOnce);
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'PHYSICALBIOASSAY'};
			var tableAttributes = {'table-id': 'bioassays'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnNewMergedDerivedBioAssay', bioassays.newMergedDerivedBioAssay);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	bioassays.initElements = function(element, autoInit)
	{
		if (autoInit == 'new-derived-bioassay')
		{
			Buttons.addClickHandler(element, bioassays.newDerivedBioAssay);
		}
	}
	
	bioassays.newMergedDerivedBioAssay = function()
	{
		if (!Table.checkIfSelected('bioassays'))
		{
			return;
		}
		Table.submitToPopup('bioassays', 'NewMergedDerivedBioAssay', 750, 500);
	}

	bioassays.newDerivedBioAssay = function(event)
	{
		var parentId = Data.int(event.currentTarget, 'item-id');
		Items.newItem('DERIVEDBIOASSAY', '&physicalbioassay_id='+parentId);
	}
	
	bioassays.validatePhysicalBioAssay = function()
	{
		var frm = document.forms['bioassay'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		else if (!Numbers.isInteger(frm.size.value))
		{
			Forms.showNotification(frm.size, "'" + frm.size.value + "' is not a valid number");
			return false;
		}
		else if (parseInt(frm.size.value) <= 0)
		{
			Forms.showNotification(frm.size, "Size must be &gt;0");
			return false;
		}
		return true;
	}

	bioassays.save = function()
	{
		var frm = document.forms['bioassay'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			Annotations.saveInheritedAnnotationsToForm(frm);
			Link.exportActions('extracts');
			frm.submit();
		}
	}

	bioassays.loadAnnotationsFrame = function()
	{
		Annotations.autoLoadEditFrame(bioassays.getProtocolId(), ItemSubtype.getSubtypeId('subtype_id'));
	}
	
	bioassays.loadInheritedAnnotationsFrame = function()
	{
		Annotations.autoLoadInheritFrame(bioassays.getParents());
	}
	
	bioassays.getProtocolId = function()
	{
		var frm = document.forms['bioassay'];
		var protocolId = 0;
		if (frm.protocol_id.length > 0 && !frm.protocol_id.disabled) 
		{
			protocolId = Math.abs(parseInt(frm.protocol_id.value));				
		}
		return protocolId;
	}
	
	bioassays.getParents = function()
	{
		var frm = document.forms['bioassay'];
		var parents = [];

		var arraySlideId = Math.abs(parseInt(frm.arrayslide_id.value));
		if (arraySlideId > 0) parents[parents.length] = 'ARRAYSLIDE:'+arraySlideId;
		var hardwareId = Math.abs(parseInt(frm.hardware_id.value));
		if (hardwareId > 0) parents[parents.length] = 'HARDWARE:'+hardwareId;

		var ids = Link.getIdsInList(frm.extracts, 'EXTRACT');
		if (ids.length > 0)
		{
			parents[parents.length] = 'EXTRACT:'+ids.join(':');
		}
		return parents;
	}

	bioassays.selectProtocol = function()
	{
		var frm = document.forms['bioassay'];
		var url = '&resetTemporary=1';
		url += ItemSubtype.createRelatedFilter('subtype_id', 'PROTOCOL');
		if (frm.protocol_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.protocol_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('PROTOCOL', 'protocol_id', 0, url);
	}
	
	bioassays.setProtocolCallback = function(event)
	{
		var frm = document.forms['bioassay'];
		var list = frm.protocol_id;
		if (list.length < 2 || list[1].value == '0')
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}
	
	bioassays.subtypeOnChange = function()
	{
		var frm = document.forms['bioassay'];
		var subtypeId = ItemSubtype.getSubtypeId('subtype_id');
		var recentInfo = ItemSubtype.getRelatedProjectDefaultAndRecentItems('PHYSICALBIOASSAY', subtypeId, ['PROTOCOL', 'HARDWARE']);
		
		ItemSubtype.updateSelectionList(frm.protocol_id, recentInfo.PROTOCOL['recent'], recentInfo.PROTOCOL['default']);
		ItemSubtype.updateSelectionList(frm.hardware_id, recentInfo.HARDWARE['recent'], recentInfo.HARDWARE['default']);
	}

	bioassays.selectHardware = function()
	{
		var frm = document.forms['bioassay'];
		var url = '&resetTemporary=1';
		url += ItemSubtype.createRelatedFilter('subtype_id', 'HARDWARE');
		if (frm.hardware_id.length > 1)
		{
			var id = Math.abs(parseInt(frm.hardware_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('HARDWARE', 'hardware_id', 0, url);
	}
	
	bioassays.setHardwareCallback = function(event)
	{
		var frm = document.forms['bioassay'];
		var list = frm.hardware_id;
		if (list.length < 2 || list[1].value == '0')
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}

	bioassays.selectArraySlide = function()
	{
		var frm = document.forms['bioassay'];
		var url = '';
		if (frm.arrayslide_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.arrayslide_id[1].value));				
			url += '&item_id='+id;
		}
		url += '&resetTemporary=1&tmpfilter:STRING:physicalBioAssay.name==';
		url += '&tmpfilter:BOOLEAN:destroyed=false';
		Dialogs.selectItem('ARRAYSLIDE', 'arrayslide_id', 0, url);
	}
	
	bioassays.setArraySlideCallback = function(event)
	{
		var frm = document.forms['bioassay'];
		if (frm.arrayslide_id.length < 2) 
		{
			frm.arrayslide_id[frm.arrayslide_id.length] = new Option();
		}
		frm.arrayslide_id[1].value = event.detail.id;
		frm.arrayslide_id[1].text = event.detail.name;
		frm.arrayslide_id.selectedIndex = 1;
		bioassays.updateSize(event.detail.id);
	}

	bioassays.arraySlideOnChange = function()
	{
		var frm = document.forms['bioassay'];
		var arraySlideId = frm.arrayslide_id.value;
		if (arraySlideId > 0) 
		{
			bioassays.updateSize(arraySlideId);
		}
	}
	
	bioassays.updateSize = function(arraySlideId)
	{
		var request = Ajax.getXmlHttpRequest();
		if (request != null)
		{
			var url = '../../lims/arrayslides/ajax.jsp?ID='+App.getSessionId();
			url += '&cmd=GetArrayDesign&item_id=' + arraySlideId;
			request.open("GET", url, true);
			Ajax.setReadyStateHandler(request, bioassays.updateSizeCallback);
			request.send(null);
		}
		return request != null;
	}
	
	bioassays.updateSizeCallback = function(request)
	{
		var frm = document.forms['bioassay'];
		var response = JSON.parse(request.responseText);
		if (response.status != 'ok')
		{
			alert(response.message);
			return false;
		}

		if (response.numArrays)
		{
			frm.size.value = response.numArrays;
		}
	}
	
	bioassays.addExtracts = function(event)
	{
		var frm = document.forms['bioassay'];
		var currentExtracts = Link.getIdsInList(frm.extracts, 'EXTRACT');
		var url = "&resetTemporary=1&exclude="+currentExtracts.join(',');
		url += ItemSubtype.createRelatedFilter('subtype_id', 'EXTRACT');
		Dialogs.selectItem('EXTRACT', event.currentTarget.id, 1, url);
	}

	bioassays.addExtractCallback = function(event)
	{
		event.detail.name += ' [-]';
		event.detail.value = ':1';
		Link.addItem('extracts', 'EXTRACT', event.detail);
		bioassays.extractsOnChange();
	}

	bioassays.extractsOnChange = function()
	{
		var frm = document.forms['bioassay'];
		var item = frm.extracts[frm.extracts.selectedIndex].item;
		if (item && item.id)
		{
			var i = item.value ? item.value.indexOf(':') : -1;
			frm.used_quantity.value = i >= 0 ? item.value.substring(0, i) : item.value;
			frm.position.value = i >= 0 ? item.value.substring(i+1) : '1';
			frm.used_quantity.focus();
		}
		else
		{
			frm.used_quantity.value = '';
			frm.position.value = '1';
		}
	}

	bioassays.usedQuantityOnBlur = function()
	{
		var frm = document.forms['bioassay'];
		var usedQuantity = frm.used_quantity.value;
		var position = frm.position.value;
		var size = parseInt(frm.size.value);
		if (position != '' && (position > size || position <= 0))
		{
			Forms.showNotification(frm.position, 'Position must be between 1 and ' + size);
			return;
		}
		if (position == '') position = 1;
		var displayQuantity = usedQuantity == '' ? '-' : usedQuantity+' µg';
		for (var i = 0; i < frm.extracts.length; i++)	// >
		{
			var option = frm.extracts[i];
			if (option.selected && option.item.id)
			{
				option.item.value = usedQuantity + ':' + position;
				var text = option.text.replace(/\[.*\]/, '['+displayQuantity+']');
				text = text.replace(/\d*\:/, position + ':');
				option.text = text; 
			}
		}
	}

	
	return bioassays;
}();

Doc.onLoad(BioAssays.initPage);
Doc.addElementInitializer(BioAssays.initElements);

